/**
  @file /ai/Diplomacy/dipcoefs.h
  @brief Header modulu s definicemi koeficientu pro diplomacii.
  
  Header modulu s definicemi koeficientu pro diplomacii.
  
  @author PZ
  @version 0.1
*/

#ifndef __PETR_ZITA_DIPLOMACY_COEFICIENTS__
#define __PETR_ZITA_DIPLOMACY_COEFICIENTS__

/** Makro pro hodnotu minimalni duvery v diplomatickem vztahu - duvera vsak muze jit i do zaporu. */
#define MINIMAL_BELIEF 0
/** Makro pro hodnotu maximalni duvery v diplomatickem vztahu. */
#define MAXIMAL_BELIEF 10000

/** Makro pro hodnotu duvery na pomezi vztahu spojenectvi a primeri. */
#define ALLY_TRUCE_BORDER ((int)(((MAXIMAL_BELIEF-MINIMAL_BELIEF)*3)/4))
/** Makro pro hodnotu duvery na pomezi vztahu primeri a nepratelstvi. */
#define TRUCE_ENEMY_BORDER ((int)((MAXIMAL_BELIEF-MINIMAL_BELIEF)/2))
/** Makro pro hodnotu duvery na pomezi vztahu nepratelstvi a valka. */
#define ENEMY_WAR_BORDER ((int)((MAXIMAL_BELIEF-MINIMAL_BELIEF)/4))

/** Makro pro defaultni hodnotu duvery u vztahu spojenectvi. */
#define DEFAULT_BELIEF_FOR_ALLY ((int)((MAXIMAL_BELIEF+ALLY_TRUCE_BORDER)/2))
/** Makro pro defaultni hodnotu duvery u vztahu primeri. */
#define DEFAULT_BELIEF_FOR_TRUCE ((int)((ALLY_TRUCE_BORDER+TRUCE_ENEMY_BORDER)/2))
/** Makro pro defaultni hodnotu duvery u vztahu nepratelstvi. */
#define DEFAULT_BELIEF_FOR_ENEMY ((int)((TRUCE_ENEMY_BORDER+ENEMY_WAR_BORDER)/2))
/** Makro pro defaultni hodnotu duvery u vztahu nepratelstvi. */
#define DEFAULT_BELIEF_FOR_WAR ((int)((ENEMY_WAR_BORDER+MINIMAL_BELIEF)/2))

/**
  Makra pro zjistovani diplomatickych vztahu.
  Jdou uzit pouze v ramci modulu diplomacie!
*/
#define IS_ALLY(dipid1,dipid2) ((getRelationshipByIDs((dipid1),(dipid2)))==RELATIONSHIP_ALLY)
#define IS_TRUCE(dipid1,dipid2) ((getRelationshipByIDs((dipid1),(dipid2)))==RELATIONSHIP_TRUCE)
#define IS_ENEMY(dipid1,dipid2) ((getRelationshipByIDs((dipid1),(dipid2)))==RELATIONSHIP_ENEMY)
#define IS_WAR(dipid1,dipid2) ((getRelationshipByIDs((dipid1),(dipid2)))==RELATIONSHIP_WAR)

/** Makra pro zjistovani odpovidajicich diplomatickych vztahu dle hodnoty duvery. */
#define BELIEF_ALLY(bel) (((bel)>=ALLY_TRUCE_BORDER) && ((bel)<=MAXIMAL_BELIEF))
#define BELIEF_TRUCE(bel) (((bel)>=TRUCE_ENEMY_BORDER) && ((bel)<ALLY_TRUCE_BORDER))
#define BELIEF_ENEMY(bel) (((bel)>=ENEMY_WAR_BORDER) && ((bel)<TRUCE_ENEMY_BORDER))
#define BELIEF_WAR(bel) ((bel)<ENEMY_WAR_BORDER) // WAR muze jit i do zaporu - nema dolni hranici


#endif


