/**
*******************************************************************************

  @file /gui/common/init_env.h
  @brief Nizkourovnova inicializace UI, rutiny pro testovani
  

  @author Vajicek
  @version 0.1
 
******************************************************************************/
#ifndef INIT_ENV_H
#define INIT_ENV_H

#include <SDL/SDL.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include "gui/GUI.h"

namespace gui{

/** Inicializuje GUI 
  
  @retval -1 chyba
  @retval 0 OK
*/
extern int InitEnv(int _w, int _h, int _fullscreen);

/** Prepina rozliseni.
  
  @param width  Sirka obrazovky.
  @param height Vyska obrazovky.
  @param bpp  Bitu na pixel.
  @param fs   Fullscreen.
  @return Chybovy kod.
*/
extern int SwitchResolution(int width, int height, int bpp, int fs);

/** Ukonci rozhrani
*/
extern void ShutdownEnv();


#ifdef DEMO

/**Testovaci uzivatelska kamera.
  
  Aby uzivatel mohl chodit po scene. 
  Klavesy:
    -A,W,D,S - pohyb
    -sipky vlevo, vpravo - otaceni kolem osy Y
    -sipky nahoru, dolu - pohyb ve smeru osy Y
  @param keyscanrate jak casto skanovat klavesy
  @retval 0 - byl stisknut escape
*/
extern int UserCamera(int keyscanrate = 30);


/** Vykresli osovy kriz.
  
  Pro testovaci ucely.
*/
extern void DrawAxis();

/** Vykresli souradnicovou mriz
*/
extern void DrawGrid(int n);


/** Nastavi projekci
*/
extern void SetupPerspective(double av,double ar,double n,double f);


/** Klavesy
*/
extern int GetKey();


#endif//DEMO

}//namespace
#endif
/*****************************************************************************/
