/**
*******************************************************************************

  @file /gui/model/reader_ase.h
  @brief Nizkourovnova knihovna pro nacitani modelu(TModel) z formatu ASE
  

  @author Vajicek
  @version 0.1
 
******************************************************************************/
#ifndef READER_ASE_H
#define READER_ASE_H

#include "gui/model/ModelRep.h"
#include "common/exc.h"

namespace gui{


/** load flag - smazat obsah modelu
  inicializovat novy
*/
#define ASE_NEW       1
/** load flag - pridat k modelu
  Nemazet predchozi obsah. Spojuje se s ASE_GEOMETRY, 
  ASE_ANIMATION, ASE_HIERARCHY, ASE_LOD, pokud nahravam 
  model z vice souboru
*/
#define ASE_APPEND    2
/** load flag - nahrat jen geometrii
  Nahraje pouze geometrii. Preskakuje animacni tagy.
*/
#define ASE_GEOMETRY  4
/** load flag - nahrat jen animace
  Nahraje pouze animace. Pouziva se pokud k jedomu 
  modelu mam vic animaci ve vice souborech.
*/
#define ASE_ANIMATION 8
/** load flag - build hierarchy
  Nahraje model a usporada do hierarchie. Pouziva se 
  pro hierarchicke modely. Animovane, atd. Komplikovanejsi
  vykreslovani. 
*/
#define ASE_HIERARCHY 16
/** load flag - Mesh only
  Nahraje pouze data a nevytvari hierarchii. 
*/
#define ASE_MESHONLY 32
/** load flag - lod level
*/
#define ASE_LOD       64

/** nacitat texturove souradnice obracene
*/
#define ASE_TEXTUREUPDOWN 256


/** transformace radianu na stupne
*/
#define ASE_DEGREES 512


/** Mozne kombinace flagu
  ASE_NEW | ASE_GEOMETRY | ASE_MESHONLY

  ASE_NEW | ASE_LOD | ASE_MESHONLY

  ==> flagy musi obsahovat 3 parametry  + lod volitelne

  [ASE_NEW/ASE_APPEND] | [ASE_GEOMETRY/ASE_ANIMATION/..] | [ASE_MESHONLY|ASE_HIERARCHY] | ASE_LOD
*/

/** load model

  @param file filename
  @param m model/scene object
  @param flag 

  @throws E_8K_ASE_READER
*/
extern void load_ASE(char* file, TModel* m, int flag);

/** load lod model

  @param file filename
  @param m model/scene object
  @param flag 
  @param dist distance of object

  @throws E_8K_ASE_READER 
*/
extern void load_ASE(char* file, TModel* m, int flag, float dist);


}//namespace
#endif
/*****************************************************************************/
