/** @file /world/useful.h
    Uiten jednoduch funkce, makra a definice bez spolenho
    zamen.
    
    @author Petr Wolf
*/

#ifndef WORLD_USEFUL_H
#define WORLD_USEFUL_H

#include <string>

#define TCL_TEMPVAR_ATTEMPTS 100

/** Nahrazeni textu.
    @param  s    text, ve kterem se bude provadet nahrazovani
    @param  co   text, ktery bude v \c s hledan
    @param  cim  tetx, kterym jsou nahrazovany vyskyty \c co v \c s.

    @return vysledny text
*/
std::string strReplace(std::string s, std::string co, std::string cim);

/** Opatreni textu zpetnymi lomitky pred uvozovkami a dalsimi 
    specialnimi znaky.
*/
std::string addSlashes(std::string s);

/** Zapis do subtagu XML 
    @param  package  ukazatel na objekt TPackage, do ktereho se maji data zapsat
    @param  data     ukazatel na data pro zapis. Objekt, ktery musi mit metodu writeToXML
    @param  tag      jmeno tagu, do ktereho se ma zapisovat
*/
#define WRITE_TO_XML(package, data, tag) { (package)->setEmpty(tag, 0, NULL); TPackSmall small((package)->getSubtag(tag, 0, NULL)); (data)->writeToXML(&small); }

/** Nacteni ze subtagu XML
    @param  package  ukazatel na objekt TPackage, ze ktereho se maji data nacist
    @param  data     ukazatel na data pro cteni. Objekt, ktery musi mit metodu readFromXML
    @param  tag      jmeno tagu, ze ktereho se cte
*/
#define READ_FROM_XML(package, data, tag) { TPackSmall small((package)->getSubtag(tag, 0, NULL)); (data)->readFromXML(&small); }

/** Ulozeni INT hodnoty do XML */
#define SET_INT(package, name, value) { (package)->setInt((int)value, name, 0, NULL); }

/** Nacteni INT hodnoty z XML */
#define GET_INT(package, name, value) { value = (int)(package)->getInt(name, 0, NULL); }

/** Pretypovani a nacteni INT hodnoty z XML */
#define GET_TINT(package, name, value, type) { value = (type)(package)->getInt(name, 0, NULL); }

/** Inicializace datoveho balicku pri zpracovani synchronizacnich informaci na klientovu.
    Makro inicializuje balicek, nacte do nej data z prostredi a z XML a pak zamkne WORLD
    pro upravy.    

    Po provedeni zmen ve WORLDu musi nasledovat PACKET_SEND
    @see PACKET_SEND
*/
#define WORLD_CLIENT_GUI_PACKET_INIT(classname) classname packet; packet.player_id = player_id; packet.readFromXML(package); world_client[i]->world->lock();

/** Odeslani datoveho balicku GUI.
    Makro nasleduje po PACKET_INIT a provedeni zmen v modulu WORLD. Odemkne svet a odesle zpravu GUI.
    @see PACKET_SEND
*/
#define WORLD_CLIENT_GUI_PACKET_SEND(msgname) world_client[i]->world->unlock(); KSendGlobalMessage(msgname, MOD_WORLD_CLIENT, MOD_GUI, &packet);

/// Prijem a zpracovani zpravy od GUI pro WORLD_CLIENT 
#define WORLD_CLIENT_SERVER_PACKET_INIT(struct_type, msgname) struct_type * packet = (struct_type *)msg; TPacket_SyncResult * result = new TPacket_SyncResult(); TPackage package(MSG_WORLD_CLIENT_SERVER); package.setInt(packet->player_id, PACKET_WORLD::player_id, 0, NULL); package.setInt(msgname, PACKET_WORLD::type, 0, NULL);

/// Odeslani zpravy z WORLD_CLIENT k WORLD_SERVER 
#define WORLD_CLIENT_SERVER_PACKET_SEND package.send(world_client[0]->world->players[packet->player_id]->data().client_data.client_id, TO_SERVER, MOD_WORLD_SERVER);

/// Kontrola, je-li hrac odesilajici zpravu na tahu
#define WORLD_CLIENT_PLAYER_OPERATIONAL world_client[0]->world->playerOperational(packet->player_id)

/// Kontrola, je-li jednotka, ktere se zprava tyka, na tahu
#define WORLD_CLIENT_UNIT_OPERATIONAL(unit_id) world_client[0]->world->unitOperational(unit_id, packet->player_id)

/// Kontrola, je-li budova, ktere se zprava tyka, na tahu
#define WORLD_CLIENT_BUILDING_OPERATIONAL(building_id) world_client[0]->world->buildingOperational(building_id, packet->player_id)

/// Kontrola, je-li hrac odesilajici zpravu na tahu
#define WORLD_SERVER_AI_PLAYER_OPERATIONAL ((world_server->state == WSS_GAME_STARTED) && world_server->world.playerOperational(packet->player_id))

/// Kontrola, je-li jednotka, ktere se zprava tyka, na tahu
#define WORLD_SERVER_AI_UNIT_OPERATIONAL(unit_id) ((world_server->state == WSS_GAME_STARTED) && world_server->world.unitOperational(unit_id, packet->player_id))

/// Kontrola, je-li budova, ktere se zprava tyka, na tahu
#define WORLD_SERVER_AI_BUILDING_OPERATIONAL(building_id) ((world_server->state == WSS_GAME_STARTED) && world_server->world.buildingOperational(building_id, packet->player_id))

/// Kontrola, je-li hrac odesilajici zpravu na tahu
#define WORLD_SERVER_CLIENT_PLAYER_OPERATIONAL ((world_server->state == WSS_GAME_STARTED) && world_server->world.playerOperational(player_id))

/// Kontrola, je-li jednotka, ktere se zprava tyka, na tahu
#define WORLD_SERVER_CLIENT_UNIT_OPERATIONAL(unit_id) ((world_server->state == WSS_GAME_STARTED) && world_server->world.unitOperational(unit_id, player_id))

/// Kontrola, je-li budova, ktere se zprava tyka, na tahu
#define WORLD_SERVER_CLIENT_BUILDING_OPERATIONAL(building_id) ((world_server->state == WSS_GAME_STARTED) && world_server->world.buildingOperational(building_id, player_id))

/// Test konce hry nebo porazeni hrace
#define WORLD_SERVER_FINAL_TEST { if (world_server->engine.finalTest()) { world_server->synchronize(); world_server->endGame(); } else { world_server->synchronize(); }}
                               
/// Zkratka na predani tahu dalsimu hraci
#define WORLD_SERVER_NEXT_TURN { TPackage package(MSG_WORLD_CLIENT_SERVER); package.setInt(MSG_NET_ACT_NEXT_TURN, PACKET_WORLD::type, 0, NULL); package.setInt(world_server->world.players.playerIdOnTurn(), PACKET_WORLD::player_id, 0, NULL); KSendGlobalMessage(MSG_WORLD_CLIENT_SERVER, MOD_WORLD_SERVER, MOD_WORLD_SERVER, &package); }

/// Zkratka pro predani tahu AI
#define WORLD_SERVER_AI_ON_TURN { TPacket_AI_ON_TURN packet_on_turn; packet_on_turn.player_id = world_server->world.players.playerIdOnTurn(); packet_on_turn.max_action_id = world_server->history.maxId(); packet_on_turn.ai_level = world_server->world.players.playerOnTurn()->data().client_data.type; KSendGlobalMessage(MSG_AI_ON_TURN, MOD_WORLD_SERVER, MOD_STRATEGY, &packet_on_turn); }

/*********************************************/
// Uzitecne makra pro informace z Worldu

#define IS_UNIT_ON_HEX(map,x,y) (((((map)->getHexAt((x),(y)))->data()).unit)!=0) // vraci bool
#define IS_UNIT_ON_HEXID(map,hexid) (((((map)->getHexById((hexid)))->data()).unit)!=0) // vraci bool

#define UNIT_ID_ON_HEX(map,x,y) ((((map)->getHexAt((x),(y)))->data()).unit) // vraci ID jednotky (int)
#define UNIT_ID_ON_HEXID(map,hexid) ((((map)->getHexById((hexid)))->data()).unit) // vraci ID jednotky (int)

#define UNIT_ON_HEX(map,living_units,x,y) (((*(living_units))[((((map)->getHexAt((x),(y)))->data()).unit)])->data()) // vraci LIVING_UNIT
#define UNIT_ON_HEXID(map,living_units,hexid) (((*(living_units))[((((map)->getHexById((hexid)))->data()).unit)])->data()) // vraci LIVING_UNIT

#define IS_BUILDING_ON_HEX(map,x,y) (((((map)->getHexAt((x),(y)))->data()).building)!=0) // vraci bool
#define IS_BUILDING_ON_HEXID(map,hexid) (((((map)->getHexById((hexid)))->data()).building)!=0) // vraci bool

#define BUILDING_ID_ON_HEX(map,x,y) ((((map)->getHexAt((x),(y)))->data()).building) // vraci ID budovy (int)
#define BUILDING_ID_ON_HEXID(map,hexid) ((((map)->getHexById((hexid)))->data()).building) // vraci ID budovy (int)

#define BUILDING_ON_HEX(map,living_buildings,x,y) (((*(living_buildings))[((((map)->getHexAt((x),(y)))->data()).building)])->data()) // vraci LIVING_BUILDING
#define BUILDING_ON_HEXID(map,living_buildings,hexid) (((*(living_buildings))[((((map)->getHexById((hexid)))->data()).building)])->data()) // vraci LIVING_BUILDING

#define VISIBILITY_FOR_PLAYER_ON_HEX(visibility_maps,playerID,x,y) ((((*(visibility_maps))[(playerID)])->getHexAt((x),(y))).getVisibility()) // vraci VISIBILITY
#define VISIBILITY_FOR_PLAYER_ON_HEXID(visibility_maps,playerID,hexid) ((((*(visibility_maps))[(playerID)])->getHexById((hexid))).getVisibility()) // vraci VISIBILITY

#define UNIT_TYPE_ON_HEX(map,rules,living_units,x,y) ((((rules)->unit_types)[(UNIT_ON_HEX((map),(living_units),(x),(y)).type)])->data()) // vraci UNIT
#define UNIT_TYPE_ON_HEXID(map,rules,living_units,hexid)  ((((rules)->unit_types)[(UNIT_ON_HEXID((map),(living_units),(hexid)).type)])->data()) // vraci UNIT

#define BUILDING_TYPE_ON_HEX(map,rules,living_buildings,x,y) ((((rules)->building_types)[(BUILDING_ON_HEX((map),(living_buildings),(x),(y)).type)])->data()) // vraci BUILDING
#define BUILDING_TYPE_ON_HEXID(map,rules,living_buildings,hexid) ((((rules)->building_types)[(BUILDING_ON_HEXID((map),(living_buildings),(hexid)).type)])->data()) // vraci BUILDING

#define TOWN_FROM_ITERATOR(citIt) (((citIt)->second)->data()) // vraci TOWN
#define KINGDOM_FROM_ITERATOR(kIt) (((kIt)->second)->data()) // vraci KINGDOM

#define AI_DATA_FOR_PLAYER(players_pointer,playerID) ((players_pointer)->ai_data->plan[(playerID)]) // vraci char* - data, ktere si serializuje AI

#define UNIT_ON_HEXID_CAN_ATTACK(map,rules,living_units,hexid) (((UNIT_TYPE_ON_HEXID((map),(rules),(living_units),(hexid))).fight_type!=FT_NONE) && (((UNIT_ON_HEXID((map),(living_units),(hexid))).state==US_IDLE) || (((UNIT_ON_HEXID((map),(living_units),(hexid))).state==US_HAS_MOVED) && (((UNIT_TYPE_ON_HEXID((map),(rules),(living_units),(hexid))).movement_and_fight_combination==MFC_ANYTHING) || ((UNIT_TYPE_ON_HEXID((map),(rules),(living_units),(hexid))).movement_and_fight_combination==MFC_MOVEMENT_FIRST))))) // vraci bool, zdali jednotka jeste muze utocit


/*********************************************/

#endif
/*****************************************************************************/
